@@ -57,8 +57,35 @@ class RemoveTracebackFilter(logging.Filter):
         return True
 
 
+class RemoveTRACEFilter(logging.Filter):
+
+    def filter(self, record: logging.LogRecord) -> bool:
+        if record.msg.startswith("TRACE"):
+            return False
+        else:
+            return True
+
+
+class PassTRACETOKENFilter(logging.Filter):
+    def __init__(self, trace_filter: str):
+        super().__init__()
+        self.trace_filter = trace_filter
+        self.prefix = "TRACE/" + self.trace_filter
+
+    def filter(self, record: logging.LogRecord) -> bool:
+        if record.msg.startswith("TRACE"):
+            if record.msg.startswith(self.prefix):
+                return True
+            else:
+                return False
+        else:
+            return True
+
+
 REMOVE_TRACEBACK_FILTER: logging.Filter = RemoveTracebackFilter()
 
+REMOVE_TRACE_FILTER: logging.Filter = RemoveTRACEFilter()
+
 
 class IdentLogRecordFactory:
     """LogRecordFactory that adds ``ident`` attribute."""
@@ -77,7 +104,13 @@ class IdentLogRecordFactory:
                 ident += "/%s" % (record.threadName or "unknown")
             if (sys.version_info >= (3, 8) and
                     record.thread == threading.get_ident()):
-                tid = threading.get_native_id()
+                try:
+                    tid = threading.get_native_id()
+                except AttributeError:
+                    # so far function not existing e.g. on SunOS
+                    # see also https://docs.python.org/3/library/threading.html#threading.get_native_id
+                    tid = None
+
         record.ident = ident  # type:ignore[attr-defined]
         record.tid = tid  # type:ignore[attr-defined]
         return record
@@ -221,18 +254,42 @@ def setup() -> None:
         logger.error("Invalid RADICALE_LOG_FORMAT: %r", format_name)
 
 
-def set_level(level: Union[int, str], backtrace_on_debug: bool) -> None:
+logger_display_backtrace_disabled: bool = False
+logger_display_backtrace_enabled: bool = False
+
+
+def set_level(level: Union[int, str], backtrace_on_debug: bool, trace_on_debug: bool = False, trace_filter: str = "") -> None:
     """Set logging level for global logger."""
+    global logger_display_backtrace_disabled
+    global logger_display_backtrace_enabled
     if isinstance(level, str):
         level = getattr(logging, level.upper())
         assert isinstance(level, int)
     logger.setLevel(level)
     if level > logging.DEBUG:
-        logger.info("Logging of backtrace is disabled in this loglevel")
+        if logger_display_backtrace_disabled is False:
+            logger.info("Logging of backtrace is disabled in this loglevel")
+            logger_display_backtrace_disabled = True
         logger.addFilter(REMOVE_TRACEBACK_FILTER)
     else:
         if not backtrace_on_debug:
-            logger.debug("Logging of backtrace is disabled by option in this loglevel")
+            if logger_display_backtrace_disabled is False:
+                logger.debug("Logging of backtrace is disabled by option in this loglevel")
+                logger_display_backtrace_disabled = True
             logger.addFilter(REMOVE_TRACEBACK_FILTER)
         else:
+            if logger_display_backtrace_enabled is False:
+                logger.debug("Logging of backtrace is enabled by option in this loglevel")
+                logger_display_backtrace_enabled = True
             logger.removeFilter(REMOVE_TRACEBACK_FILTER)
+        if trace_on_debug:
+            if trace_filter != "":
+                logger.debug("Logging messages starting with 'TRACE/%s' enabled", trace_filter)
+                logger.addFilter(PassTRACETOKENFilter(trace_filter))
+                logger.removeFilter(REMOVE_TRACE_FILTER)
+            else:
+                logger.debug("Logging messages starting with 'TRACE' enabled")
+                logger.removeFilter(REMOVE_TRACE_FILTER)
+        else:
+            logger.debug("Logging messages starting with 'TRACE' disabled")
+            logger.addFilter(REMOVE_TRACE_FILTER)
